package models

import "encoding/base64"

// NewDocument creates an empty Document
func NewDocument(mimetype string) *Document {
	return &Document{
		Type: mimetype,
	}
}

// SetMetadata sets a metadata
func (m *Document) SetMetadata(name, value string) {
	if m.Metadata == nil {
		m.Metadata = make(Metadata)
	}
	m.Metadata[name] = value
}

// UpdateMetadata sets several metadata entries at once
func (m *Document) UpdateMetadata(entries Metadata) *Document {
	if m.Metadata == nil {
		m.Metadata = make(Metadata)
	}
	for key, value := range entries {
		m.Metadata[key] = value
	}
	return m
}

// SetBytes sets the document content
func (m *Document) SetBytes(data []byte) *Document {
	m.BodyFormat = "base64"
	m.Body = base64.StdEncoding.EncodeToString(data)
	return m
}

// Bytes is the document content as a slice of bytes
func (m *Document) Bytes() ([]byte, error) {
	switch m.BodyFormat {
	case "":
		fallthrough
	case "base64":
		return base64.StdEncoding.DecodeString(m.Body)
	case "text":
		return []byte(m.Body), nil
	default:
		return []byte(m.Body), nil
	}
}

// SetString sets the document content
func (m *Document) SetString(data string) *Document {
	m.SetBytes([]byte(data))
	return m
}

// String is the document content as a string
func (m *Document) String() (string, error) {
	switch m.BodyFormat {
	case "":
		fallthrough
	case "base64":
		b, err := base64.StdEncoding.DecodeString(m.Body)
		if err != nil {
			return "", err
		}
		return string(b), nil
	case "text":
		return m.Body, nil
	default:
		return m.Body, nil
	}
}
